/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 
package org.apache.taglibs.benchmark;

import java.io.*;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 *
 *  A tag handler for the &lt;benchmark:duration&gt; tag, which determines
 *  how long it takes to compute its body (minus the sections of its body
 *  enclosed in &lt;benchmark:exclude&gt; ... &lt;/benchmark:exclude&gt;).
 *
 *  @version 0.90
 *  @author Shawn Bayern
 */

public class Duration extends BodyTagSupport {

    private long repeat = 1;		// number of repetitions
    private boolean output = false;	// do we output our contents?

    private long start, end;		// our own start and end times
    private boolean running;		// is our timer currently running?
    private long exclude;		// time to subtract from timer
    private long exStart;		// start of exclusion timer (tmp)
    
    public void release() {
	init();
    }

    public int doStartTag() throws JspException {
	start = System.currentTimeMillis();
	running = true;

	return EVAL_BODY_TAG;
    }

    public int doEndTag() throws JspException {
	try {
	    // first, stop the clock
	    end = System.currentTimeMillis();
	    running = false;

	    // next, output the body contents if appropriate
	    if (output)
		getPreviousOut().write(getBodyContent().getString());

	    // finally, output duration
	    pageContext.getOut().print(end - start - exclude);

	} catch (IOException ex) {
	    throw new JspTagException(ex.getMessage());
	}
	return EVAL_PAGE;
    }

    public int doAfterBody() throws JspException {
	if (--repeat > 0)
	    return EVAL_BODY_TAG;
	else
	    return SKIP_BODY;
    }

    /** Temporarily pauses our timer, which must already be running. */
    public void pauseTimer() throws JspTagException {
	if (!running) {
	    /*
	     * Display an error message customized for the common case,
	     * since this situation is possible but erroneous.  I've
	     * somewhat arbitrarily caught the error here instead of in
	     * <exclude> itself on the premise that if other tags are
	     * plugged in, we can localize this handling in the parent class.
	     * This may be stupid, but it's 5:16 in the morning, so what
	     * do I know?
	     */
	    throw new JspTagException(
		"can't &lt;exclude&gt; an already excluded section "
		+ "or otherwise pause a paused &lt;duration&gt; timer");
	}

	// start a separate exclusion timer
	exStart = System.currentTimeMillis();
	running = false;
    }

    /** Restarts our timer, which must already be paused. */
    public void restartTimer() throws JspTagException {
	if (running) {
	    // shouldn't happen unless new custom tags are added later
	    throw new JspTagException(
		"unexpected use of Duration.restartTimer() on active clock");
	}

	// end the current exclusion timer and compute exclusion
	long exEnd = System.currentTimeMillis();
	exclude += exEnd - exStart;
	running = true;
    }

    /** Initializes the timer. */
    private void init() {
	start = end = exclude = exStart = 0;
	repeat = 1;
	running = output = false;
    }


    // accessors

    public void setRepeat(long x) throws JspTagException {
	if (x < 0)
	    throw new JspTagException(
		"'repeat' in &lt;benchmark:duration&gt; cannot be negative");
	repeat = x;
    }

    public long getRepeat() {
	return repeat;
    }

    public void setOutput(boolean x) {
	output = x;
    }

    public boolean getOutput() {
	return output;
    }
}
