/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.taglibs.cache;

import javax.servlet.jsp.PageContext;

/**
 * <p>Contains static methods that can be used to save and retrieve
 * information used to retrieve appropriate caches and to 
 * configure the runtime behavior of the Cache Taglib.</p>
 *
 * @author Shawn Bayern
 */

public class CacheUtil {

    //*********************************************************************
    // Constants

    private static final String CACHE_PAGE =
      "org.apache.taglibs.cache.page";
    private static final String CACHE_REQUEST =
      "org.apache.taglibs.cache.request";
    private static final String CACHE_SESSION =
      "org.apache.taglibs.cache.session";
    private static final String CACHE_APPLICATION =
      "org.apache.taglibs.cache.application";

    private static final String CACHES_PREFIX = "caches.";
    private static final String LIFETIME = "lifetime";
    private static final String SIZE = "size";

    private static final int DEFAULT_SIZE = 65536;
    private static final int DEFAULT_LIFETIME = 300;

    //*********************************************************************
    // Public methods

    /**
     * Retrieves the size of the cache, given a particular scope and
     * a context.
     */ 
    public static int getCacheSize(int scope, PageContext pageContext) {
      String attribute = getAttributeName(scope, SIZE);
      Number n = (Number) pageContext.getAttribute(attribute, scope);
      if (n == null)
        return DEFAULT_SIZE;
      else
        return n.intValue(); 
    }

    /**
     * Retrieves the lifetime of items in the cache, given a particular
     * scope and a context.
     */ 
    public static int getCacheLifetime(int scope, PageContext pageContext) {
      String attribute = getAttributeName(scope, LIFETIME);
      Number n = (Number) pageContext.getAttribute(attribute, scope);
      if (n == null)
        return DEFAULT_LIFETIME;
      else
        return n.intValue();
    }

    /**
     * Sets a particular cache size to use for new caches in the
     * given scope.
     */
    public static void setCacheSize(int size, int scope, PageContext ctx) {
     String attribute = getAttributeName(scope, SIZE);
     ctx.setAttribute(attribute, new Integer(size), scope);
    }

    /**
     * Sets a particular cache lifetime to use for new caches in the
     * given scope.
     */
    public static void setCacheLifetime(
        int lifetime, int scope, PageContext ctx) {
      String attribute = getAttributeName(scope, LIFETIME);
      ctx.setAttribute(attribute, new Integer(lifetime), scope);
    }

    /**
     * Retrieves (and creates if necessary) an appropriate LRUCache
     * given a scope, name, and context.
     */
    public static LRUCache getCache(int scope, String name, PageContext ctx) {
      String att = getAttributePrefixForScope(scope) + CACHES_PREFIX + name;
      LRUCache l = (LRUCache) ctx.getAttribute(att, scope);
      if (l == null) {
        l = new LRUCache(
          getCacheSize(scope, ctx), getCacheLifetime(scope, ctx));
        ctx.setAttribute(att, l, scope);
      }
      return l;
    }

    /**
     * Invalidates an entire cache
     */
    public static void invalidateCache(
        int scope, String name, PageContext pageContext) {
      String att = getAttributePrefixForScope(scope) + CACHES_PREFIX + name;
      pageContext.removeAttribute(att, scope);
    }

    /**
     * Invalidates an individual cache entry (key)
     */
    public static void invalidateCachedItem(
        int scope, String name, String key, PageContext pageContext) {
      LRUCache l = getCache(scope, name, pageContext);
      if (l == null)
        return;						// nothing to do
      l.remove(key);
    }


    //*********************************************************************
    // Private utility methods

    private static String getAttributeName(int scope, String variable) {
      return getAttributePrefixForScope(scope) + variable;
    }

    private static String getAttributePrefixForScope(int scope) {
      switch(scope) {
      case PageContext.PAGE_SCOPE:
        return CACHE_PAGE + ".";
      case PageContext.REQUEST_SCOPE:
        return CACHE_REQUEST + ".";
      case PageContext.SESSION_SCOPE:
        return CACHE_SESSION + ".";
      case PageContext.APPLICATION_SCOPE:
        return CACHE_APPLICATION + ".";
      default:
        throw new IllegalArgumentException("unknown scope");
      }
    }    
}
