/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.dbtags.connection;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.JspTagException;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

/**
 * <p>JSP tag connection, used to get a
 * java.sql.Connection object from the DriverManager.</p>
 *
 * <p>JSP Tag Lib Descriptor
 * <pre>
 * &lt;name>connection&lt;/name>
 * &lt;tagclass>org.apache.taglibs.dbtags.connection.ConnectionTag&lt;/tagclass>
 * &lt;bodycontent>JSP&lt;/bodycontent>
 * &lt;teiclass>org.apache.taglibs.dbtags.connection.ConnectionTEI&lt;/teiclass>
 * &lt;info>Opens a connection based on either a url in the body of the tag
 * or by using the "datasource" tag attribute to reference to a
 * javax.sql.DataSource page attribute.  driver (optional),
 * userid (optional), and password (optional) are also set in the
 * body of the tag.&lt;/info>
 *   &lt;attribute>
 *     &lt;name>id&lt;/name>
 *     &lt;required>true&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>jndiName&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>dataSource</name>
 *     &lt;required>false</required>
 *     &lt;rtexprvalue>false</rtexprvalue>
 *   &lt;/attribute>
 * </pre>
 *
 * @author Morgan Delagrange
 */
public class ConnectionTag extends BodyTagSupport {

  private String _dataSourceName = null;
  private String _url      = null;
  private String _driver   = null;
  private String _userId   = null;
  private String _password = null;
  private String _jndiName = null;

  /**
   * Set the name of a javax.sql.DataSource page attribute
   * which can create a database connection
   *
   * @param dataSourceName
   *               name of a javax.sql.DataSource page attribute
   */
  public void setDataSource(String dataSourceName) {
    _dataSourceName = dataSourceName;
  }

  /**
   * URL of the database to access.
   *
   * @param url    database URL
   */
  public void setUrl(String url) {
    _url = url;
  }

  /**
   * java.sql.Driver for the database.  This method is
   * optional if you have already loaded the Driver in
   * the JVM.
   *
   * @param driver class name of the java.sql.Driver
   */
  public void setDriver(String driver) {
    _driver = driver;
  }

  /**
   * jndi named datasource used for connecting to the database via a jndi lookup
   *
   * @param jndiName jndi name for the jdbc datasourcer
   */
  public void setJndiName(String jndiName) {
    _jndiName = jndiName;
  }

  /**
   * User id for the database.  Optional if the user id
   * is already encoded in the URL, or if it is not
   * required by the database.
   *
   * @param userId user id for the database.
   */
  public void setUserId(String userId) {
    _userId = userId;
  }

  /**
   * Password for the database.  Optional if the
   * password is already encoded in the URL, or if
   * it is not required by the database.
   *
   * @param password password for the given user id
   */
  public void setPassword(String password) {
    _password = password;
  }

  public int doStartTag() throws JspTagException {
    return EVAL_BODY_TAG;
  }

  public int doAfterBody() throws JspTagException
  {
    // Automatically trim the contents of this tag
    try {
      getPreviousOut().write(getBodyContent().getString().trim());
    } catch (IOException e) {
      throw new JspTagException(e.toString());
    }
    return EVAL_PAGE;
  }

  public int doEndTag() throws JspTagException{

    DataSource dataSource = null;

    try {
      if (_driver != null) {
        Class.forName(_driver);
      }

      Connection conn = null;

      if (_jndiName != null) {
	try {
          Context ctx = new InitialContext();
	  dataSource = (DataSource)ctx.lookup(_jndiName);
	} catch (NamingException ne) {
	    throw new JspTagException(ne.toString());
	}
	if (_userId != null) {
          conn = dataSource.getConnection(_userId, _password);
        } else {
          conn = dataSource.getConnection();
        }
      } else if (_dataSourceName != null) {
        dataSource = (DataSource) pageContext.findAttribute(_dataSourceName);
        if (_userId != null) {
          conn = dataSource.getConnection(_userId, _password);
        } else {
          conn = dataSource.getConnection();
        }
      } else if (_url != null) {
        if (_userId != null) {
          conn = DriverManager.getConnection(_url, _userId, _password);
        } else {
          conn = DriverManager.getConnection(_url);
        }
      } else {
        throw new JspTagException("Cannot connect to database, no database URL or DataSource.");
      }

      pageContext.setAttribute(getId(),conn);
    } catch (SQLException e) {
      throw new JspTagException(e.toString());
    } catch (ClassNotFoundException e) {
      throw new JspTagException("Driver class '" + _driver + "' not found\n" +
                                e.toString());
    }

    return EVAL_PAGE;
  }

  public void release() {
    _dataSourceName = null;
    _url      = null;
    _driver   = null;
    _userId   = null;
    _password = null;
  }

}
