
/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.preparedstatement;

import java.lang.reflect.Field;
import java.sql.PreparedStatement;
import java.sql.Types;

import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
 * Base class for all the setter tags without bodies 
 * in the preparedstatement package.
 * 
 * @author Morgan Delagrange
 */
public class BaseSetterTag extends TagSupport {
  
  protected int _position = 1;
  protected String _attributeName = null;
  
  /**
   * Sets the column number of the prepared statement.
   * 
   * @param position column index
   */
  public void setPosition(int position) {
    _position = position;
  }
  
  /**
   * Name of the page attribute that will be assigned
   * to the statement.
   * 
   * @param attributeName
   *               attribute name
   */
  public void setName(String attributeName) {
    _attributeName = attributeName;
  }
      
  /**
   * Gets the page attribute for the tag
   * 
   * @param name   name of the attribute
   * @return the page attribute
   * @exception JspTagException
   *                   thrown when the page attribute does not exist
   */
  protected Object getAttribute(String name) 
      throws JspTagException {
    Object object = pageContext.getAttribute(name);
    
    if (object == null) {
      throw new JspTagException("attribute " + name + " does not exist");
    }
      
    return object;
  }

  /**
   * get the PreparedStatement from the enclosing tag
   * 
   * @return the PreparedStatement
   * @exception JspTagException
   *                   thrown if no PreparedStatement exists
   */
  protected PreparedStatement getPreparedStatement() 
      throws JspTagException {
    try {
      PreparedStatementImplTag stmtTag = 
      (PreparedStatementImplTag) findAncestorWithClass(this, Class.forName("org.apache.taglibs.dbtags.preparedstatement.PreparedStatementImplTag"));
      return stmtTag.getPreparedStatement();
    } catch (ClassNotFoundException e) {
      throw new JspTagException(e.toString());
    }
  }
  
  /**
   * Perfoms reflection to translate a String name to one of
   * the int values of java.sql.Types
   * 
   * @param fieldName name of the field in java.sql.Types
   * @return int value of the sql type
   * @exception JspTagException
   *                   thrown if the sql type does not exist, or if
   *                   reflection fails
   */
  protected int getSqlTypeForFieldName(String fieldName) 
      throws JspTagException {
    try {
      Class typesClass = Class.forName("java.sql.Types");
      Field typeField  = typesClass.getField(fieldName);
      return typeField.getInt(null);
    } catch (ClassNotFoundException e) {
      throw new JspTagException(e.toString());
    } catch (NoSuchFieldException e) {
      throw new JspTagException("Illegal argument: setnull tag could not find = '" + 
                         fieldName + "' field in the java.sql.Types class" + e.toString());
    } catch (IllegalAccessException e) {
      throw new JspTagException(e.toString());
    }
  }

  public void release() {
    _position = 1;
    _attributeName = null;
  }
}
