/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.resultset;

import java.sql.Types;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Locale;

import javax.servlet.jsp.JspTagException;

/**
 * Base tag for getDate, getTime, and getTimestamp
 * 
 * @author Morgan Delagrange
 * @author Marius Scurtescu
 */
public class BaseDateTimeGetterTag extends BaseGetterTag {

  protected static HashMap _mapDateTimeStyle = new HashMap (4);
  protected String _format = null;

  static
  {
    _mapDateTimeStyle.put ("FULL", new Integer (DateFormat.FULL));
    _mapDateTimeStyle.put ("LONG", new Integer (DateFormat.LONG));
    _mapDateTimeStyle.put ("MEDIUM", new Integer (DateFormat.MEDIUM));
    _mapDateTimeStyle.put ("SHORT", new Integer (DateFormat.SHORT));
  }

  public void setFormat (String strFormat) {
    _format = strFormat;
  }

  /**
   * Returns a formatted Date as a String, or null if the 
   * database row was null
   * 
   * @param position position of the column in the ResultSet
   * @param nType    one of the following java.sql.Types: DATE, TIME, TIMESTAMP
   * @return a formatted String, or null
   * @exception JspTagException
   */
  protected String getDateAsString(int position, int nType) throws JspTagException {

    ResultSet rset = getResultSet();

    String string = null;
    try {

      switch (nType) {
        case Types.DATE:
          {
            java.sql.Date date = rset.getDate (position);

            if (date != null) {
              if (_format == null)
                return date.toString ();

              DateFormat fmt = getDateFormat (nType, _format, getLocale (_locale));

              string = fmt.format (date);
            }
            break;
          }
        case Types.TIME:
          {
            java.sql.Time time = rset.getTime (position);

            if (time != null) {
              if (_format == null)
                return time.toString ();

              DateFormat fmt = getDateFormat (nType, _format, getLocale (_locale));

              string = fmt.format (time);
            }
            break;
          }
        case Types.TIMESTAMP:
          {
            java.sql.Timestamp times = rset.getTimestamp (position);

            if (times != null) {
              if (_format == null)
                return times.toString ();

              DateFormat fmt = getDateFormat (nType, _format, getLocale (_locale));

              string = fmt.format (times);
            }
            break;
          }
        default:
          throw new JspTagException();
      }
    } catch (SQLException e) {
      throw new JspTagException(e.toString());
    }

    return string;

  }

  private DateFormat getDateFormat (int nType, String strFormat, Locale loc) throws JspTagException
  {
    switch (nType) {
      case Types.DATE:
        {
          if (!_mapDateTimeStyle.containsKey (strFormat))
            return new SimpleDateFormat (strFormat, loc);

          Integer iDateStyle = (Integer)_mapDateTimeStyle.get (strFormat);
          return DateFormat.getDateInstance (iDateStyle.intValue(), loc);
        }

      case Types.TIME:
        {
          if (!_mapDateTimeStyle.containsKey (strFormat))
            return new SimpleDateFormat (strFormat, loc);

          Integer iTimeStyle = (Integer)_mapDateTimeStyle.get (strFormat);
          return DateFormat.getTimeInstance (iTimeStyle.intValue(), loc);
        }

      case Types.TIMESTAMP:
        {
          Integer iDateStyle, iTimeStyle;
          int pos = strFormat.indexOf (",");

          if (pos == -1) {
            if (!_mapDateTimeStyle.containsKey (strFormat))
              return new SimpleDateFormat (strFormat, loc);

            iDateStyle = iTimeStyle = (Integer)_mapDateTimeStyle.get (strFormat);
          } else {
            String strDateStyle = strFormat.substring (0, pos);
            String strTimeStyle = strFormat.substring (pos + 1);

            if (!_mapDateTimeStyle.containsKey (strDateStyle) || !_mapDateTimeStyle.containsKey (strTimeStyle))
              return new SimpleDateFormat (strFormat, loc);

            iDateStyle = (Integer)_mapDateTimeStyle.get (strDateStyle);
            iTimeStyle = (Integer)_mapDateTimeStyle.get (strTimeStyle);
          }

          return DateFormat.getDateTimeInstance (iDateStyle.intValue (), iTimeStyle.intValue (), loc);
        }

      default:
        throw new JspTagException ("Not a Date/Time JDBC type: " + nType);
    }
  }


}
