/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */
package org.apache.taglibs.dbtags.resultset;

import org.apache.taglibs.dbtags.statement.StatementTag;

import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.io.IOException;

import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.PageContext;


/**
 * <p>JSP tag resultSet, executes the query and loops through the results
 * for the enclosing statement or preparedstatement tag.  The body of
 * this tag is executed once per row in the resultset.  The optional
 * "loop" attribute, which default to true, specifies whether to execute
 * the tag body once per row "true", or to simply assign the ResultSet
 * to the page attribute specified by "id". The optional "name" and "scope"
 * attributes can be used to retrieve a resultset (or rowset) from context.</p>
 *
 * <p>JSP Tag Lib Descriptor
 * <pre>
 * &lt;name>resultSet&lt;/name>
 * &lt;tagclass>org.apache.taglibs.dbtags.resultset.ResultSetTag&lt;/tagclass>
 * &lt;teiclass>org.apache.taglibs.dbtags.connection.ResultSetTEI&lt;/teiclass>
 * &lt;bodycontent>JSP&lt;/bodycontent>
 * &lt;info>JSP tag resulset, executes the query and loops through the results
 * for the enclosing statement or preparedstatement tag.  The body of
 * this tag is executed once per row in the resultset.  The optional
 * "loop" attribute, which default to true, specifies whether to execute
 * the tag body once per row "true", or to simply assign the ResultSet
 * to the page attribute specified by "id". The optional "name" and "scope"
 * attributes can be used to retrieve a resultset (or rowset) from context.<&lt;/info>
 *   &lt;attribute>
 *     &lt;name>id&lt;/name>
 *     &lt;required>true&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>loop&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>name&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>true&lt;/rtexprvalue>
 *   &lt;/attribute>
 *   &lt;attribute>
 *     &lt;name>scope&lt;/name>
 *     &lt;required>false&lt;/required>
 *     &lt;rtexprvalue>false&lt;/rtexprvalue>
 *   &lt;/attribute>
 * </pre>
 *
 * @author Morgan Delagrange
 * @author Ted Husted
 * @author Craig McClanahan
 * @see org.apache.taglibs.dbtags.statement.StatementImplTag
 * @see org.apache.taglibs.dbtags.statement.QueryTag
 * @see org.apache.taglibs.dbtags.preparedstatement.PreparedStatementImplTag
 */
public class ResultSetTag extends BodyTagSupport{

  private Statement _statement = null;
  private ResultSet _rset = null;
  private boolean _shouldLoop = true;
  private String _name = null;
  private String _scope = null;
  private int _rowCount = 0;

  /**
   * Locate and return the specified bean, from an optionally specified
   * scope, in the specified page context.  If no such bean is found,
   * return <code>null</code> instead.  If an exception is thrown, it will
   * have already been saved via a call to <code>saveException()</code>.
   *
   * @param pageContext Page context to be searched
   * @param name Name of the bean to be retrieved
   * @param scope Scope to be searched (page, request, session, application)
   *  or <code>null</code> to use <code>findAttribute()</code> instead
   *
   * @exception JspException if an invalid scope name
   *  is requested
   */
  protected Object lookup(PageContext pageContext,
      String name, String scope) throws JspTagException {

      Object bean = null;
      if (scope == null)
          bean = pageContext.findAttribute(name);
      else if (scope.equalsIgnoreCase("page"))
          bean = pageContext.getAttribute(name, PageContext.PAGE_SCOPE);
      else if (scope.equalsIgnoreCase("request"))
          bean = pageContext.getAttribute(name, PageContext.REQUEST_SCOPE);
      else if (scope.equalsIgnoreCase("session"))
          bean = pageContext.getAttribute(name, PageContext.SESSION_SCOPE);
      else if (scope.equalsIgnoreCase("application"))
          bean =
              pageContext.getAttribute(name, PageContext.APPLICATION_SCOPE);
      else {
          JspTagException e = new JspTagException("Invalid scope " + scope);
          throw e;
      }
      return (bean);

   }

  /**
   * Name of the bean that contains the rowset to process.
   */


  public void setName(String name) {
       _name = name;
   }

  public void setScope(String scope) {
       _scope = scope;
   }

  public void setLoop(boolean shouldLoop) {
    _shouldLoop = shouldLoop;
  }

  public ResultSet getResultSet() {
    return _rset;
  }

  public int doStartTag() throws JspTagException {

      try {
            // if name property given, retrieve resultset from context
            // may also be a related class, like RowSet
        if (_name!=null) {
          _rset = (ResultSet) lookup(pageContext, _name, _scope);
        }
        else {
            StatementTag _stmtTag =
                (StatementTag) findAncestorWithClass(this,
                    Class.forName("org.apache.taglibs.dbtags.statement.StatementTag"));
            _rset = _stmtTag.executeQuery();
        }

        pageContext.setAttribute(getId(), _rset);

        if (_shouldLoop == false) {
            return EVAL_BODY_TAG;
        }

        if (_rset.next() == false) {
          setTotalRowCount(_rowCount);
          return SKIP_BODY;
        }

      ++_rowCount;
      setTotalRowCount(_rowCount);

    } catch (ClassNotFoundException e) {
      throw new JspTagException(e.toString());
    } catch (SQLException e) {
      throw new JspTagException(e.toString());
    }

    return EVAL_BODY_TAG;
  }

  public int doEndTag() throws JspTagException{
    pageContext.removeAttribute(getId());
    try {
      if (getBodyContent() != null && getPreviousOut() != null) {
        getPreviousOut().write(getBodyContent().getString());
      }
    } catch (IOException e) {
      throw new JspTagException(e.toString());
    } finally {
      try {
        _rset.close();
      }
      catch (SQLException e) {
        // it's not fatal if the result set cannot be closed
        e.printStackTrace();
      }
    }

    // we have to call this guy manually now
    // with the spec clarification
    release();

    return EVAL_PAGE;
  }

  public int doAfterBody() throws JspTagException{

    if (_shouldLoop == false) {
      return EVAL_PAGE;
    }

    try {
      if (_rset.next() == true) {
        ++_rowCount;
        setTotalRowCount(_rowCount);
        return EVAL_BODY_TAG;
      }
    } catch (SQLException e) {
      throw new JspTagException(e.toString());
    }

    setTotalRowCount(_rowCount);
    return EVAL_PAGE;
  }

  public void release() {
    _statement = null;
    _rset = null;
    _shouldLoop = true;
    _name = null;
    _scope = null;
    _rowCount = 0;
  }

  protected void setTotalRowCount(int rowCount) {
    pageContext.setAttribute("org.apache.taglibs.dbtags.resultset.rowcount",
                              new Integer(rowCount));
  }

}
