/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 
package org.apache.taglibs.input;

import java.util.*;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 *
 *  This class implements the &lt;input:select&gt; tag, which presents a
 *  &lt;select&gt; form element.
 *
 *  @version 0.90
 *  @author Shawn Bayern
 *  @author Lance Lavandowska
 */

public class Select extends TagSupport {

    private String name;        // name of the select element
    private String dVal;        // default value if none is found
    private String[] dValArray;     // our multiple default values
    private Map attributes;     // attributes of the <select> element
    private Map options;        // what are our options? :)
    private String attributesText; // attributes of the <input> element as text
    private String beanId;      // bean id to get default values from
    private boolean multiple;   // select multiple
    private String size;           // select size
    private List optionLabels;   // a list of option labels
    private List optionValues;   // a list of option values
    
    private HashMap chosen;     // chosen options (created in doStartTag)

    public int doStartTag() throws JspException {
        try {
            // sanity check
            if (name == null || name.equals(""))
                throw new JspTagException("invalid null or empty 'name'");

            // Store beanId in a local variable as we change it
            String beanId = this.beanId;

            // Get default beanId
            if ( beanId == null ) {
                beanId = Util.defaultFormBeanId( this );
            }
            else if ( beanId.length() == 0 ) {
                // An empty beanId means, do not use any bean - not even default
                beanId = null;
            }

            // get what we need from the page
            ServletRequest req = pageContext.getRequest();
            JspWriter out = pageContext.getOut();

            // start building up the tag
            out.print("<select name=\"" + Util.quote(name) + "\" ");

            // include any attributes we've got here
            Util.printAttributes(out, attributes);
            if ( attributesText != null ) {
                out.print( attributesText + " " );
            }
            
            if ( multiple ) {
                out.print( "multiple=\"true\" " );
            }
            if ( size != null ) {
                out.print( "size=\"" + Util.quote(size) + "\" " );
            }

            // end the starting tag
            out.println(">");

            /*
             * Print out our options, selecting one or more if appropriate.
             * If there are multiple selections but the page doesn't call
             * for a <select> that accepts them, ignore the selections.
             * This is preferable to throwing a JspException because the
             * (end) user can control input, and we don't want the user
             * causing exceptions in our application.
             */

            String[] selected;
            
            // Use selected from the bean, if available
            String [] beanValues = ( beanId != null ? Util.beanPropertyValues(
                pageContext.findAttribute( beanId ), name ) : null );
            if ( beanValues != null ) {
                selected = beanValues;
            }
            else {
                // get the current selection from the request
                selected = req.getParameterValues(name);
                if ( selected == null ) {
                    // Use defaults
                    if ( dValArray != null && dVal != null ) {
                        selected = new String[ dValArray.length + 1 ];
                        selected[0] = dVal;
                        System.arraycopy( dValArray, 0, selected, 1, dValArray.length );
                    }
                    else if ( dValArray != null ) {
                        selected = dValArray;
                    }
                    else if ( dVal != null ) {
                        selected = new String[] { dVal };
                    }
                }
            }
            
            if (selected != null && selected.length > 1 &&
                    ((attributes == null || !attributes.containsKey("multiple")) && !multiple))
                selected = null;

            // load up the selected values into a hash table for faster access
            // and for option tags to access
            chosen = new HashMap();
            if (selected != null) {
                for (int i = 0; i < selected.length; i++) {
                    if ( selected[i] != null ) {
                        chosen.put(selected[i], null);
                    }
                }
            }


            // actually print the <option> tags
            if ( optionLabels != null ) {
                // Print out using the optionLabels
                int n = optionLabels.size();
                for ( int i = 0; i < n; i++ ) {
                    Object oKey = optionLabels.get( i );
                    Object oVal = ( options != null ? options.get(oKey)
                                                    : ( optionValues != null ? optionValues.get( i ) : oKey ) );

                    outputOption( out, oKey, oVal );
                }
            }
            else if (options != null) {
                Iterator i = options.keySet().iterator();
                while (i.hasNext()) {
                    Object oKey = i.next();
                    Object oVal = options.get(oKey);

                    outputOption( out, oKey, oVal );
                }
            }

        } catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
        return EVAL_BODY_INCLUDE;
    }
    
    private void outputOption( JspWriter out, Object oKey, Object oVal ) throws java.io.IOException {
        String key = oKey.toString();
        /*
         * Convert the value to a String if it is not already.
         */
        String value = ( oVal != null ? oVal.toString() : null );
        
        if (value == null)
            value = key;        // use key if value is null

        out.print("<option");
        if (!value.equals(key))
            out.print(" value=\"" + Util.quote(value) + "\"");
        /*
         * This may look confusing: we match the VALUE of
         * this option pair with the KEY of the 'chosen' Map
         * (We want to match <option>s on values, not keys.)
         */
        if (chosen.containsKey(value))
            out.print(" selected=\"true\"");
        out.print(">");
        out.print(Util.quote(key));
        out.println("</option>");
    }
    
    public int doEndTag() throws JspException {
        try {
            JspWriter out = pageContext.getOut();
            out.print( "</select>" );
        } catch (Exception ex) {
            throw new JspTagException(ex.getMessage());
        }
        return EVAL_PAGE;
    }
    
    public void setName(String x) {
        name = x;
    }

    public void setAttributes(Map x) {
        attributes = x;
    }
    
    public void setAttributesText( String x ) {
        attributesText = x;
    }
    
    public void setBean( String x ) {
        beanId = x;
    }

    public void setMultiple( boolean x ) {
        multiple = x;
    }
    
    public void setSize( String x ) {
        size = x;
    }
    
    public void setDefault(String x) {
        dVal = x;
    }

    public void setDefaults(String[] x) {
	dValArray = x;
    }
    
    public void setDefaults(Map x) {
        dValArray = new String[ x.size() ];
        Iterator it = x.keySet().iterator();
        int i = 0;
        while ( it.hasNext() ) {
            dValArray[i++] = it.next().toString();
        }
    }

    public void setDefaults(Collection c) {
        dValArray = new String[ c.size() ];
        Iterator it = c.iterator();
        int i = 0;
        while ( it.hasNext() ) {
            dValArray[i++] = it.next().toString();
        }
    }

    public void setOptions(Map x) {
        options = x;
    }

    public void setOptionLabels(List x) {
        optionLabels = x;
    }

    public void setOptionValues(List x) {
        optionValues = x;
    }
    
    public HashMap getChosen() {
        return chosen;
    }
    
    /** Getter for property name.
     * @return Value of property name.
     */
    public String getName() { return name; }
    
    /** Getter for property default.
     * @return Value of property default.
     */
    public String getDefault() { return dVal; }
    
    /** Getter for property bean.
     * @return Value of property bean.
     */
    public String getBean() { return beanId; }
    
    /** Getter for property attributesText.
     * @return Value of property attributesText.
     */
    public String getAttributesText() { return attributesText; }
    
    /** Getter for property attributes.
     * @return Value of property attributes.
     */
    public Map getAttributes() { return attributes; }
    
    /** Getter for property defaults.
     * @return Value of property defaults.
     */
    public String[] getDefaults() { return dValArray; }
    
    /** Getter for property multiple.
     * @return Value of property multiple.
     */
    public boolean isMultiple() { return multiple; }
    
    /** Getter for property optionLabels.
     * @return Value of property optionLabels.
     */
    public List getOptionLabels() { return optionLabels; }
    
    /** Getter for property optionValues.
     * @return Value of property optionValues.
     */
    public List getOptionValues() { return optionValues; }
    
    /** Getter for property options.
     * @return Value of property options.
     */
    public Map getOptions() { return options; }
    
    /** Getter for property size.
     * @return Value of property size.
     */
    public String getSize() { return size; }
    
}
