/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.io;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Reader;
import java.io.Writer;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/** A collection of bean and introspection helper methods.
  * 
  * @author <a href="mailto:james.strachan@metastuff.com">James Strachan</a>
  * @version $Revision: 1.1 $
  */
public class BeanHelper {

    protected static final Object[] NULL_ARGUMENTS = {};
    
    /** Gets the value of the given property
      * 
      * @param bean is the JavaBean which contains the property
      * @param propertyName is the name of the property to set
      *
      * @return the value of the given property
      */
    public static Object getProperty( Object bean, String propertyName ) throws JspException {
        try {
            PropertyDescriptor descriptor
                = getPropertyDescriptor( bean, propertyName );
            Method method = descriptor.getReadMethod();
            return method.invoke( bean, NULL_ARGUMENTS );
        }
        catch (Exception e) {
            throw new JspException( 
                "Failed to get property: " + propertyName 
                + " on bean: " +  bean + ". Exception: " + e );
        }
    }
    
    /** Sets the value of the given property
      * 
      * @param bean is the JavaBean which contains the property
      * @param propertyName is the name of the property to set
      * @param value is the value of the property to set
      *
      * @return true if the property was set else false if it could not be set
      */
    public static void setProperty( Object bean, String propertyName, Object value ) throws JspException {
        try {
            PropertyDescriptor descriptor
                = getPropertyDescriptor( bean, propertyName );
            Method method = descriptor.getWriteMethod();
            Object[] arguments = { value };
            method.invoke( bean, arguments );
        }
        catch (Exception e) {
            throw new JspException( 
                "Failed to set property: " + propertyName 
                + " on bean: " +  bean + ". Exception: " + e );
        }
    }
    
    /** @return a PropertyDescriptor for the given bean and property name
      */
    protected static PropertyDescriptor getPropertyDescriptor( 
        Object bean, 
        String propertyName 
    ) throws JspException {
        try {
            Class beanClass = bean.getClass();
            BeanInfo beanInfo = Introspector.getBeanInfo( beanClass );

            // Its a shame the JDK doesn't have a helper method on BeanInfo for this
            PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();
            if ( descriptors != null ) {
                int size = descriptors.length;
                for ( int i = 0; i < size; i++ ) {
                    PropertyDescriptor descriptor = descriptors[i];
                    String name = descriptor.getName();
                    if ( propertyName.equals( name ) ) {
                        return descriptor;
                    }
                }
            }
            throw new JspException( 
                "No such property: " + propertyName + " on bean: " +  bean 
            );
        }
        catch (IntrospectionException e) {
            throw new JspException( 
                "Failed to instrospect bean: " +  bean + ". Exception: " + e 
            );
        }
    }
}
