/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.taglibs.io;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Reader;
import java.io.Writer;

import javax.servlet.jsp.tagext.Tag;
import javax.servlet.jsp.tagext.TagSupport;

/** A collection of helper methods for pipelining data between tags.
  * 
  * @author <a href="mailto:james.strachan@metastuff.com">James Strachan</a>
  * @version $Revision: 1.2 $
  */
public class PipeHelper {
    
    /** Size of the buffer used when piping a stream */
    protected static final int BUFFER_SIZE = 64 * 1024;
    
    /** Temporarily disable the closing of Readers */
    protected static final boolean CLOSE_READER = false;

    
    /** Uses the standard pipelining guidelines to produce a Reader
      * instance for the given object. 
      * The object can be either a Reader, an InputStream or a String.
      *
      * @return a Reader for the given input or null if the instance
      * could not be converted into a Reader.
      */
    public static Reader getReader(Object input) {
        if ( input instanceof Reader ) {
            return (Reader) input;
        }
        else if ( input instanceof InputStream ) {
            return new InputStreamReader( (InputStream) input );
        }
        else if ( input instanceof String ) {
            return new StringReader( (String) input );
        }
        return null;
    }
    
    /** Uses the standard pipelining guidelines to produce a Writer
      * instance for the given object. 
      * The object can be either a Writer or an InputStream.
      *
      * @return a Writer for the given input or null if the instance
      * could not be converted into a Writer.
      */
    public static Writer getWriter(Object output) {
        if ( output instanceof Writer ) {
            return (Writer) output;
        }
        else if ( output instanceof OutputStream ) {
            return new OutputStreamWriter( (OutputStream) output );
        }
        return null;
    }
    
    
    /** Pipes all the input from the given reader to the given writer. 
      * The reader will be closed whether an error occurs or not. 
      * Though the writer is left open.
      */
    public static void pipe(Reader reader, Writer writer) throws IOException {
        pipe( reader, writer, new char[ BUFFER_SIZE / 2 ] );
    }
    
    /** Pipes all the input from the given reader to the given writer. 
      * The reader will be closed whether an error occurs or not. 
      * Though the writer is left open.
      */
    public static void pipe(Reader reader, Writer writer, char[] buffer) throws IOException {
        try {
            while (true) {
                int size = reader.read( buffer );
                if ( size <= 0 ) {
                    return;
                }
                writer.write( buffer, 0, size );
            }
        }
        finally {
            if ( CLOSE_READER ) {
                try {
                    reader.close();
                }
                catch (Exception e) {
                }
            }
        }
    }
    
    /** Pipes all the input to the given output.
      * The input stream will be closed whether an error occurs or not. 
      * Though the output stream is left open is left open.
      */
    public static void pipe(InputStream input, OutputStream output) throws IOException {
        pipe( input, output, new byte[ BUFFER_SIZE ] );
    }
    
    
    /** Pipes all the input to the given output.
      * The input stream will be closed whether an error occurs or not. 
      * Though the output stream is left open is left open.
      */
    public static void pipe(InputStream input, OutputStream output, byte[] buffer) throws IOException {
        try {
            while (true) {
                int size = input.read( buffer );
                if ( size <= 0 ) {
                    return;
                }
                output.write( buffer, 0, size );
            }
        }
        finally {
            try {
                input.close();
            }
            catch (IOException e) {
            }
        }
    }
}
