package com.taglib.weblog;

import com.taglib.util.DbConnectionFactory;
import com.taglib.authentication.UserBean;
import com.taglib.authentication.DbUserBase;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.text.SimpleDateFormat;

import java.sql.SQLException;
import java.text.ParseException;

public class DbWeblogDataManager {
  private DbConnectionFactory connectionFactory = DbConnectionFactory.getInstance();

  static private DbWeblogDataManager INSTANCE = new DbWeblogDataManager();
  static public DbWeblogDataManager getInstance () {
    return INSTANCE;
  }
  static private SimpleDateFormat DB_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");

  private DbWeblogDataManager () {}

  public Entry fetchEntry (long id) throws WeblogException {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_ENTRY);
      stmt.setLong(1, id);
      rset = stmt.executeQuery();
      if (rset.next()) {
        Entry entry = new Entry();
        entry.setId(id);
        entry.setCreated(rset.getTimestamp(1));
        entry.setLastModified(rset.getTimestamp(2));
        entry.setTitle(rset.getString(3));
        entry.setText(rset.getString(4));
        long authorId = rset.getLong(5);
        entry.setAuthor(DbUserBase.fetch(authorId));
        return entry;
      }
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to fetch entry.", e);
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
    return null;
  }

  public List fetchEntries (Date forDate) throws WeblogException {
    String dateString = DB_DATE_FORMAT.format(forDate);
    return fetchEntries(dateString);
  }
  public List fetchEntries (String dateString) throws WeblogException {
    ArrayList entries = new ArrayList();
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_DATE_ENTRIES);
      stmt.setString(1, dateString);
      rset = stmt.executeQuery();
      while (rset.next()) {
        long id = rset.getLong(1);
        entries.add(fetchEntry(id));
      }
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to fetch entries for " + dateString + ".", e);
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
    return entries;
  }

  public List fetchRecentDates (int count) throws WeblogException {
    ArrayList dates = new ArrayList();
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_FETCH_RECENT_DATES);
      stmt.setInt(1, count);
      rset = stmt.executeQuery();
      while (rset.next()) {
        String dateString = rset.getString(1);
        try {
          dates.add(DB_DATE_FORMAT.parse(dateString));
        }
        catch (ParseException e) {
          throw new WeblogException("Unable to parse date from db.", e);
        }
      }
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to fetch recent dates.", e);
    }
    finally {
      if (rset != null) try { rset.close(); } catch (SQLException e) {}
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
    return dates;
  }

  public void postEntry (Entry newEntry) throws WeblogException {
    Connection conn = null;
    PreparedStatement stmt = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_POST_ENTRY);
      stmt.setString(1, newEntry.getTitle());
      stmt.setString(2, newEntry.getText());
      stmt.setLong(3, newEntry.getAuthor().getId());
      stmt.execute();
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to post new entry.", e);
    }
    finally {
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
  }
  public void updateEntry (Entry entry) throws WeblogException {
    Connection conn = null;
    PreparedStatement stmt = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_UPDATE_ENTRY);
      stmt.setString(1, entry.getTitle());
      stmt.setString(2, entry.getText());
      stmt.setLong(3, entry.getId());
      stmt.execute();
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to update entry.", e);
    }
    finally {
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
  }
  public void deleteEntry (Entry entry) throws WeblogException {
    Connection conn = null;
    PreparedStatement stmt = null;
    try {
      conn = connectionFactory.getConnection();
      stmt = conn.prepareStatement(SQL_DELETE_ENTRY);
      stmt.setLong(1, entry.getId());
      stmt.execute();
    }
    catch (SQLException e) {
      throw new WeblogException("Unable to delete entry.", e);
    }
    finally {
      if (stmt != null) try { stmt.close(); } catch (SQLException e) {}
      if (conn != null) try { conn.close(); } catch (SQLException e) {}
    }
  }

  final static private String SQL_FETCH_ENTRY =
    "SELECT created, last_modified, title, entry, author FROM blog WHERE id = ?";
  final static private String SQL_POST_ENTRY =
    "INSERT INTO blog (created, last_modified, title, entry, author) VALUES (now(), now(), ?, ?, ?)";
  final static private String SQL_UPDATE_ENTRY =
    "UPDATE blog SET title = ?, entry = ?, last_modified = now() WHERE id = ?";
  final static private String SQL_DELETE_ENTRY =
    "DELETE FROM blog WHERE id = ?";
  final static private String SQL_FETCH_RECENT_DATES =
    "SELECT date_format(created, '%Y-%m-%d') FROM blog GROUP BY date_format(created, '%Y-%m-%d') DESC LIMIT ?";
  final static private String SQL_FETCH_DATE_ENTRIES =
    "SELECT id FROM blog WHERE date_format(created, '%Y-%m-%d') = ?";
}
